/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { NavigationE2EComponent } from 'apps/alfa-e2e/src/components/navigation/navigation.e2e.component';
import { VorgangListE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-list.e2e.component';
import { VorgangSearchE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-search.e2e.component';
import { VorgangViewsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-views.e2e.component';
import { VorgangE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { dropCollections, dropSearchIndex } from 'apps/alfa-e2e/src/support/cypress-helper';
import { enterWith, exist, haveText, notExist } from 'apps/alfa-e2e/src/support/cypress.util';
import { getUserSabine, loginAsSabine } from 'apps/alfa-e2e/src/support/user-util';
import { buildVorgang, createVorgang, initSearchIndex, initVorgaenge, objectIds } from 'apps/alfa-e2e/src/support/vorgang-util';

describe('Navigation', () => {
	const mainPage: MainPage = new MainPage();

	const header: HeaderE2EComponent = mainPage.getHeader();
	const vorgangSearch: VorgangSearchE2EComponent = header.getVorgangSearch();

	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

	const navigation: NavigationE2EComponent = mainPage.getNavigation();

	const views: VorgangViewsE2EComponent = mainPage.getViews();

	const vorgang: VorgangE2E = createVorgang();
	const vorgangNotBeFiltered: VorgangE2E = { ...buildVorgang(objectIds[0], 'vorgangNotBeFiltered') };
	const vorgangAssigned: VorgangE2E = { ...buildVorgang(objectIds[1], 'vorgangAssigned'), assignedTo: getUserSabine().id };
	const vorgangAssignedNotBeFiltered: VorgangE2E = { ...buildVorgang(objectIds[2], 'vorgangAssignedNotBeFiltered'), assignedTo: getUserSabine().id };

	const searchString: string = 'NotBeFiltered';

	before(() => {
		initVorgaenge([vorgang, vorgangNotBeFiltered, vorgangAssigned, vorgangAssignedNotBeFiltered]);
		initSearchIndex([vorgang, vorgangNotBeFiltered, vorgangAssigned, vorgangAssignedNotBeFiltered]);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
		navigation.getAlleFilter().click();
		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
		dropSearchIndex();
	})

	describe('navigation item myVorgaenge', () => {

		it('should exists', () => {
			exist(navigation.getMeineVorgaengeFilter());
		})
	})

	describe('navigation item allVorgaenge', () => {

		it('should exists', () => {
			exist(navigation.getAlleFilter());
		})
	})

	describe('click on my vorgaenge', () => {

		it('should hide not to user assigned vorgaenge', () => {
			navigation.getMeineVorgaengeFilter().click();
			waitForSpinnerToDisappear();

			notExist(vorgangList.getListItem(vorgang.name).getRoot());
			notExist(vorgangList.getListItem(vorgangNotBeFiltered.name).getRoot());
			exist(vorgangList.getListItem(vorgangAssigned.name).getRoot());
			exist(vorgangList.getListItem(vorgangAssignedNotBeFiltered.name).getRoot());
		})

		it('should hide not matching vorgaenge after do search', () => {
			doSearch(searchString);
			waitForSpinnerToDisappear();

			notExist(vorgangList.getListItem(vorgang.name).getRoot());
			notExist(vorgangList.getListItem(vorgangNotBeFiltered.name).getRoot());
			notExist(vorgangList.getListItem(vorgangAssigned.name).getRoot());
			exist(vorgangList.getListItem(vorgangAssignedNotBeFiltered.name).getRoot());
		})
	})

	describe('click on all vorgaenge', () => {

		it('should show all vorgaenge', () => {
			navigation.getAlleFilter().click();
			waitForSpinnerToDisappear();
			views.getVorgangList().getRoot().click();

			waitForSpinnerToDisappear();

			exist(vorgangList.getListItem(vorgang.name).getRoot());
			exist(vorgangList.getListItem(vorgangNotBeFiltered.name).getRoot());
			exist(vorgangList.getListItem(vorgangAssigned.name).getRoot());
			exist(vorgangList.getListItem(vorgangAssignedNotBeFiltered.name).getRoot());
		})

		it('should hide not matching vorgaenge after do search', () => {
			doSearch(searchString);
			waitForSpinnerToDisappear();

			notExist(vorgangList.getListItem(vorgang.name).getRoot());
			exist(vorgangList.getListItem(vorgangNotBeFiltered.name).getRoot());
			notExist(vorgangList.getListItem(vorgangAssigned.name).getRoot());
			exist(vorgangList.getListItem(vorgangAssignedNotBeFiltered.name).getRoot());
		})
	})

	describe('click on Suche view', () => {

		it('and alle filter should show empty list', () => {
			vorgangSearch.getClearButton().click();

			waitForSpinnerToDisappear();

			views.getSuche().getRoot().click();

			waitForSpinnerToDisappear();

			notExist(vorgangList.getListItem(vorgang.name).getRoot());
			notExist(vorgangList.getListItem(vorgangNotBeFiltered.name).getRoot());
			notExist(vorgangList.getListItem(vorgangAssigned.name).getRoot());
			notExist(vorgangList.getListItem(vorgangAssignedNotBeFiltered.name).getRoot());

			//TODO sauber implementieren
			haveText(cy.getTestElement('empty-list-text'), 'Bitte geben Sie einen Suchbegriff ein.');
		})

		it('and meine filter should show empty list', () => {
			navigation.getMeineVorgaengeFilter().click();

			waitForSpinnerToDisappear();

			notExist(vorgangList.getListItem(vorgang.name).getRoot());
			notExist(vorgangList.getListItem(vorgangNotBeFiltered.name).getRoot());
			notExist(vorgangList.getListItem(vorgangAssigned.name).getRoot());
			notExist(vorgangList.getListItem(vorgangAssignedNotBeFiltered.name).getRoot());

			//TODO sauber implementieren
			haveText(cy.getTestElement('empty-list-text'), 'Bitte geben Sie einen Suchbegriff ein.');
		})
	})

	function doSearch(searchString: string): void {
		enterWith(vorgangSearch.getInput(), searchString);
	}
})