/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { PostfachMailE2EComponent } from 'apps/alfa-e2e/src/components/postfach/postfach-mail.e2e.component';
import { PostfachMailPage } from 'apps/alfa-e2e/src/page-objects/postfach-mail.component.po';
import { createPostfachNachrichtAttachedItem, createPostfachNachrichtReplyItem, initVorgangAttachedItem } from 'apps/alfa-e2e/src/support/vorgang-attached-item-util';
import { CyHttpMessages } from 'cypress/types/net-stubbing';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { HttpMethodE2E } from '../../../model/util';
import { VorgangE2E } from '../../../model/vorgang';
import { PostfachMailItemE2E, VorgangAttachedItemE2E } from '../../../model/vorgang-attached-item';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { countDownloadFiles, deleteDownloadFolder, dropCollections, interceptWithResponse } from '../../../support/cypress-helper';
import { exist, notExist } from '../../../support/cypress.util';
import { LinkRelE2E } from '../../../support/linkrels';
import { removeLinkFromResource } from '../../../support/tech.util';
import { loginAsSabine } from '../../../support/user-util';
import { buildVorgang, createVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';


describe('Postfach Nachrichten', () => {
	const vorgangList: VorgangListE2EComponent = new MainPage().getVorgangList();
	const vorgangPage: VorgangPage = new VorgangPage();
	const postfachMailContainer: PostfachMailE2EComponent = vorgangPage.getPostfachMailcontainer();
	const postfachMailPage: PostfachMailPage = new PostfachMailPage();

	const vorgang: VorgangE2E = createVorgang();
	const vorgangWithReply: VorgangE2E = { ...buildVorgang(objectIds[0], 'VorgangWithReply') };

	const postfachMailReply: PostfachMailItemE2E = createPostfachNachrichtReplyItem();
	const postfachNachrichtAttachedItem: VorgangAttachedItemE2E = { ...createPostfachNachrichtAttachedItem(objectIds[1], objectIds[0]), item: postfachMailReply };

	before(() => {
		initVorgaenge([vorgang, vorgangWithReply]);
		initVorgangAttachedItem([postfachNachrichtAttachedItem]);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('navigate to vorgangDetail', () => {

		beforeEach(() => {
			interceptWithResponse(HttpMethodE2E.GET, '*/vorgangs/*', (req) => modifyResponse(req));
		})

		function modifyResponse(req: CyHttpMessages.IncomingHttpRequest): void {
			req.reply(res => res.send(removeLinkFromResource(res.body, LinkRelE2E.POSTFACH_NACHRICHTEN)));
		}

		it('navigate to vorgangDetail', () => {
			vorgangList.getListItem(vorgang.name).getRoot().click();
			waitForSpinnerToDisappear();

			exist(vorgangPage.getVorgangDetailHeader().getRoot());
		})

		it('should hide postfach nachrichten area if no postfach is configured', () => {
			notExist(vorgangPage.getPostfachMailcontainer().getRoot());
		})
	})

	describe('download nachrichten as pdf', () => {

		describe('by download button on vorgang main page', () => {

			it('should be visible as icon button', () => {
				vorgangPage.getSubnavigation().getBackButton().click();
				vorgangList.getListItem(vorgangWithReply.name).getRoot().click();
				waitForSpinnerToDisappear();

				exist(postfachMailContainer.getDownloadButtonWithLabel());
			})

			it('should have 1 file in download folder after click on download', () => {
				deleteDownloadFolder();
				postfachMailContainer.getDownloadButtonWithLabel().click();
				waitForSpinnerToDisappear();

				countDownloadFiles().then((count) => {
					expect(count).to.eq(1);
				});
			})
		})

		describe('by download button on nachrichten details page', () => {

			it ('should show an overview of all nachrichten after clicking on nachricht', () => {
				postfachMailContainer.getList().click();
				waitForSpinnerToDisappear();

				exist(postfachMailPage.getRoot());
			})

			it ('should exist a pdf download button', () => {
				exist(postfachMailPage.getDownloadButton());
			})

			it('should have 1 file in download folder after click on download', () => {
				deleteDownloadFolder();
				postfachMailPage.getDownloadButton().click();
				waitForSpinnerToDisappear();

				countDownloadFiles().then((count) => {
					expect(count).to.eq(1);
				});
			})
		})

	})
})