/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UserSettingsE2EComponent } from 'apps/alfa-e2e/src/components/user-settings/user-settings.component.e2e';
import { VorgangListE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { isChecked, isNotChecked } from 'apps/alfa-e2e/src/support/angular.util';
import { dropCollections, reload } from 'apps/alfa-e2e/src/support/cypress-helper';
import { exist } from 'apps/alfa-e2e/src/support/cypress.util';
import { loginAsSabine } from 'apps/alfa-e2e/src/support/user-util';
import { createVorgang, initVorgang } from 'apps/alfa-e2e/src/support/vorgang-util';

describe('User Settings', () => {
	const mainPage: MainPage = new MainPage();
	const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
	const header: HeaderE2EComponent = mainPage.getHeader();
	const userSettings: UserSettingsE2EComponent = header.getUserSettings();
	const vorgang: VorgangE2E = createVorgang();

	before(() => {
		initVorgang(vorgang);

		loginAsSabine();

		waitForSpinnerToDisappear();
		exist(vorgangList.getRoot());
	})

	after(() => {
		dropCollections();
	})

	describe('click on settings icon', () => {

		before(() => {
			userSettings.getRoot().click();
		})

		it('should show notificationsSendsFor toggle', () => {
			exist(userSettings.getEmailBenachrichtigung().getRoot());
		})

		it('should show darkMode toggle', () => {
			exist(userSettings.getDarkMode().getRoot());
		})
	})

	describe('click on notificationSendsFor toggle', () => {

		it('should have initial unchecked toggle', () => {
			isNotChecked(userSettings.getEmailBenachrichtigung().getToggle());
		})

		it('should switch toggle status', () => {
			userSettings.getEmailBenachrichtigung().getToggle().click();

			isChecked(userSettings.getEmailBenachrichtigung().getToggle());
		})

		it('should be loaded after page reload', () => {
			reload();

			userSettings.getRoot().click();
			isChecked(userSettings.getEmailBenachrichtigung().getToggle());
		})
	})

	describe('click on darkMode toggle', () => {

		it('should have initial unchecked toggle', () => {
			isNotChecked(userSettings.getDarkMode().getToggle());
		})

		it('should switch toggle status', () => {
			userSettings.getDarkMode().getToggle().click();

			isChecked(userSettings.getDarkMode().getToggle());
		})
	})
})