package de.ozgcloud.alfa.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Map;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;

public class FormDataExportAdjusterTest {

	@Nested
	class TestAdjustFormDataForExport {

		private final Map<String, Object> formData = Map.of("key", "value");

		@Test
		void shouldCallReplaceValueKeys() {
			try (MockedStatic<FormDataExportAdjuster> adjusterMock = mockStatic(FormDataExportAdjuster.class)) {
				adjusterMock.when(() -> FormDataExportAdjuster.adjustFormDataForExport(any())).thenCallRealMethod();

				FormDataExportAdjuster.adjustFormDataForExport(formData);

				adjusterMock.verify(() -> FormDataExportAdjuster.replaceValueKeys(formData));
			}
		}
	}

	@Nested
	class TestReplaceValueKeys {

		@Test
		void shouldPreserveValueKeyWithoutParent() {
			Map<String, Object> formData = Map.of("value", "Kiel");

			var formDataWithoutValueKeys = FormDataExportAdjuster.replaceValueKeys(formData);

			assertThat(formDataWithoutValueKeys).isEqualTo(formData);
		}

		@Test
		void shouldEliminateValueKeyWithoutSiblings() {
			Map<String, Object> formData = Map.of("ort", Map.of("value", "Kiel"));
			Map<String, Object> expected = Map.of("ort", "Kiel");

			var formDataWithoutValueKeys = FormDataExportAdjuster.replaceValueKeys(formData);

			assertThat(formDataWithoutValueKeys).isEqualTo(expected);
		}

		@Test
		void shouldPreserveValueKeyIfHasSiblings() {
			Map<String, Object> formData = Map.of("ort", Map.of("value", "Kiel", "value2", "Kiel"));

			var formDataWithoutValueKeys = FormDataExportAdjuster.replaceValueKeys(formData);

			assertThat(formDataWithoutValueKeys).isEqualTo(formData);
		}

		@Test
		void shouldPreserveNonValueKeys() {
			Map<String, Object> formData = Map.of("empfangendestelle",
					Map.of("value", Map.of("OrganisationseinheitenAuswahl", Map.of("value", "123456"))));
			Map<String, Object> expected = Map.of("empfangendestelle", Map.of("OrganisationseinheitenAuswahl", "123456"));

			var formDataWithoutValueKeys = FormDataExportAdjuster.replaceValueKeys(formData);

			assertThat(formDataWithoutValueKeys).isEqualTo(expected);
		}
	}
}
